// paravelview.cpp
//
// A Qt-based application for managing reverse VNC sessions.
//   All Rights Reserved.
//

#include <stdlib.h>

#include <QApplication>
#include <QCloseEvent>
#include <QMessageBox>
#include <QStringList>

#include "cmdswitch.h"
#include "paravelview.h"

//
// Icons
//
#include "../icons/paravelview-22x22.xpm"
#include "../icons/paravelview-150x150.xpm"

MainWidget::MainWidget(QWidget *parent)
  : QWidget(parent)
{
  view_process=NULL;

  //
  // Process Command Switches
  //
  CmdSwitch *cmd=new CmdSwitch("paravelview",PARAVELVIEW_USAGE);
  for(unsigned i=0;i<cmd->keys();i++) {
    if(!cmd->processed(i)) {
      fprintf(stderr,"padpointd: unrecognized option\n");
      exit(256);
    }
  }

  setWindowTitle(tr("Paravel Systems Support Tool"));
  setWindowIcon(QPixmap(paravelview_22x22_xpm));
  setMinimumSize(sizeHint());
  setMaximumSize(sizeHint());

  //
  // Get Display Setting
  //
  if(getenv("DISPLAY")!=NULL) {
    view_display=getenv("DISPLAY");
  }
  if(getenv("HOME")!=NULL) {
    view_homedir=getenv("HOME");
  }

  //
  // Fonts
  //
  QFont label_font("helvetica",12,QFont::Bold);
  label_font.setPixelSize(12);
  QFont button_font("helvetica",14,QFont::Bold);
  button_font.setPixelSize(14);

  //
  // Hostname
  //
  view_hostname_label=new QLabel(tr("Hostname:"),this);
  view_hostname_label->setFont(label_font);
  view_hostname_label->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
  view_hostname_edit=new QLineEdit(this);
  connect(view_hostname_edit,SIGNAL(textChanged(const QString &)),
	  this,SLOT(hostnameChangedData(const QString &)));

  //
  // Port
  //
  view_port_label=new QLabel(tr("Port:"),this);
  view_port_label->setFont(label_font);
  view_port_label->setAlignment(Qt::AlignRight|Qt::AlignVCenter);
  view_port_spin=new QSpinBox(this);
  view_port_spin->setRange(1,0xFFFF);
  view_port_spin->setValue(5500);

  //
  // Logo
  //
  view_logo_label=new QLabel(this);
  view_logo_label->setPixmap(QPixmap(paravelview_150x150_xpm));

  view_phone_label_label=new QLabel(tr("Phone"),this);
  view_phone_label_label->setFont(label_font);
  view_phone_value_label=new QLabel("+1-(877) 447-2728",this);

  view_email_label_label=new QLabel(tr("E-Mail"),this);
  view_email_label_label->setFont(label_font);
  view_email_value_label=new QLabel("support@paravelsystems.com",this);

  //
  // Connect Button
  //
  view_connect_button=new QPushButton(tr("Connect"),this);
  view_connect_button->setFont(button_font);
  connect(view_connect_button,SIGNAL(clicked()),this,SLOT(connectData()));
  view_connect_button->setDisabled(true);

  //
  // Process Cleanup Timer
  //
  view_process_timer=new QTimer(this);
  view_process_timer->setSingleShot(true);
  connect(view_process_timer,SIGNAL(timeout()),this,SLOT(cleanupProcessData()));
}


QSize MainWidget::sizeHint() const
{
  return QSize(400,225);
}


QSizePolicy MainWidget::sizePolicy() const
{
  return QSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
}


void MainWidget::hostnameChangedData(const QString &str)
{
  view_connect_button->setDisabled(str.isEmpty());
}


void MainWidget::connectData()
{
  QStringList args;

  if(view_process==NULL) {
    view_process=new QProcess(this);
    if(!view_homedir.isEmpty()) {
      view_process->setStandardOutputFile(view_homedir+"/paravel_log.txt");
      view_process->setStandardErrorFile(view_homedir+"/paravel_log.txt");
    }
    connect(view_process,SIGNAL(finished(int,QProcess::ExitStatus)),
	    this,SLOT(vncFinishedData(int,QProcess::ExitStatus)));
    args.push_back("-nopw");
    args.push_back("-rfbport");
    args.push_back("6038");
    if(!view_display.isEmpty()) {
      args.push_back("-display");
      args.push_back(view_display);
    }
    args.push_back("-connect");
    args.push_back(view_hostname_edit->text()+
		   QString().sprintf(":%u",view_port_spin->value()));
    view_process->start("x11vnc",args);
    view_connect_button->setText(tr("Disconnect"));
    view_hostname_edit->setReadOnly(true);
    view_port_spin->setRange(view_port_spin->value(),view_port_spin->value());
  }
  else {
    view_process->kill();
  }
}


void MainWidget::vncFinishedData(int exit_code,QProcess::ExitStatus exit_status)
{
  view_process_timer->start(0);
}


void MainWidget::cleanupProcessData()
{
  delete view_process;
  view_process=NULL;
  view_connect_button->setText(tr("Connect"));
  view_hostname_edit->setReadOnly(false);
  view_port_spin->setRange(1,0xFFFF);
}


void MainWidget::closeEvent(QCloseEvent *e)
{
  if(view_process!=NULL) {
    if(QMessageBox::question(this,tr("Paravel View"),
			     tr("Closing will hang up the support connection!")+
			     "\n"+tr("Still close?"),
			     QMessageBox::No|QMessageBox::Yes)!=
       QMessageBox::Yes) {
      e->ignore();
      return;
    }
    view_process->kill();
    qApp->processEvents();
  }
  qApp->quit();
}


void MainWidget::resizeEvent(QResizeEvent *e)
{
  view_hostname_label->setGeometry(10,10,70,24);
  view_hostname_edit->setGeometry(85,10,size().width()-95,24);

  view_port_label->setGeometry(10,37,70,24);
  view_port_spin->setGeometry(85,37,100,24);

  view_logo_label->setGeometry(10,64,150,150);

  view_phone_label_label->setGeometry(170,75,size().width()-180,20);
  view_phone_value_label->setGeometry(170,95,size().width()-180,20);

  view_email_label_label->setGeometry(170,125,size().width()-180,20);
  view_email_value_label->setGeometry(170,145,size().width()-180,20);

  view_connect_button->
    setGeometry(size().width()-120,size().height()-55,110,50);
}


int main(int argc,char *argv[])
{
  QApplication a(argc,argv);
  
  MainWidget *w=new MainWidget();
  w->show();
  return a.exec();
}
